# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import (
    command_completion_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2,
)


class CommandCompletionServiceStub(object):
    """Allows clients to observe the status of their submissions.
    Commands may be submitted via the Command Submission Service.
    The on-ledger effects of their submissions are disclosed by the Transaction Service.

    Commands may fail in 2 distinct manners:

    1. Failure communicated in the gRPC error of the submission.
    2. Failure communicated in a Completion.

    Only successfully submitted commands may produce a completion event.

    Clients that do not receive a successful completion about their submission MUST NOT assume that it was successful.
    Clients SHOULD subscribe to the CompletionStream before starting to submit commands to prevent race conditions.

    Interprocess tracing of command submissions may be achieved via Zipkin by filling out the ``trace_context`` field.
    The server will return a child context of the submitted one, (or a new one if the context was missing) on both the Completion and Transaction streams.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CompletionStream = channel.unary_stream(
            "/com.daml.ledger.api.v1.CommandCompletionService/CompletionStream",
            request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamRequest.SerializeToString,
            response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamResponse.FromString,
        )
        self.CompletionEnd = channel.unary_unary(
            "/com.daml.ledger.api.v1.CommandCompletionService/CompletionEnd",
            request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndRequest.SerializeToString,
            response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndResponse.FromString,
        )


class CommandCompletionServiceServicer(object):
    """Allows clients to observe the status of their submissions.
    Commands may be submitted via the Command Submission Service.
    The on-ledger effects of their submissions are disclosed by the Transaction Service.

    Commands may fail in 2 distinct manners:

    1. Failure communicated in the gRPC error of the submission.
    2. Failure communicated in a Completion.

    Only successfully submitted commands may produce a completion event.

    Clients that do not receive a successful completion about their submission MUST NOT assume that it was successful.
    Clients SHOULD subscribe to the CompletionStream before starting to submit commands to prevent race conditions.

    Interprocess tracing of command submissions may be achieved via Zipkin by filling out the ``trace_context`` field.
    The server will return a child context of the submitted one, (or a new one if the context was missing) on both the Completion and Transaction streams.
    """

    def CompletionStream(self, request, context):
        """Subscribe to command completion events.
        Errors:
        - ``UNAUTHENTICATED``: if the request does not include a valid access token
        - ``PERMISSION_DENIED``: if the claims in the token are insufficient to perform a given operation
        - ``NOT_FOUND``: if the request does not include a valid ledger id or if the ledger has been pruned before ``begin``
        - ``INVALID_ARGUMENT``: if the payload is malformed or is missing required fields
        - ``OUT_OF_RANGE``: if the absolute offset is not before the end of the ledger
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CompletionEnd(self, request, context):
        """Returns the offset after the latest completion.
        Errors:
        - ``UNAUTHENTICATED``: if the request does not include a valid access token
        - ``PERMISSION_DENIED``: if the claims in the token are insufficient to perform a given operation
        - ``NOT_FOUND``: if the request does not include a valid ledger id
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_CommandCompletionServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "CompletionStream": grpc.unary_stream_rpc_method_handler(
            servicer.CompletionStream,
            request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamRequest.FromString,
            response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamResponse.SerializeToString,
        ),
        "CompletionEnd": grpc.unary_unary_rpc_method_handler(
            servicer.CompletionEnd,
            request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndRequest.FromString,
            response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "com.daml.ledger.api.v1.CommandCompletionService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class CommandCompletionService(object):
    """Allows clients to observe the status of their submissions.
    Commands may be submitted via the Command Submission Service.
    The on-ledger effects of their submissions are disclosed by the Transaction Service.

    Commands may fail in 2 distinct manners:

    1. Failure communicated in the gRPC error of the submission.
    2. Failure communicated in a Completion.

    Only successfully submitted commands may produce a completion event.

    Clients that do not receive a successful completion about their submission MUST NOT assume that it was successful.
    Clients SHOULD subscribe to the CompletionStream before starting to submit commands to prevent race conditions.

    Interprocess tracing of command submissions may be achieved via Zipkin by filling out the ``trace_context`` field.
    The server will return a child context of the submitted one, (or a new one if the context was missing) on both the Completion and Transaction streams.
    """

    @staticmethod
    def CompletionStream(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            "/com.daml.ledger.api.v1.CommandCompletionService/CompletionStream",
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionStreamResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CompletionEnd(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/com.daml.ledger.api.v1.CommandCompletionService/CompletionEnd",
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_command__completion__service__pb2.CompletionEndResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0


from typing import Dict, List, Sequence

from ..damlast.daml_lf_1 import DottedName, Module, ModuleRef
from ..damlast.util import module_name, package_ref


class ModuleHierarchy:
    START = "START"
    ITEM = "ITEM"
    END = "END"

    def __init__(self, package_id, module_name: "Sequence[str]" = ()):
        self.ref = ModuleRef(package_id, DottedName(module_name))
        self._items = dict()  # type: Dict[str, ModuleHierarchy]
        self._modules = list()  # type: List[Module]

    def __getitem__(self, item: str):
        """
        Retrieve the sub-:class:`ModuleBuilder` prefixed by a name.
        :param item:
        :return:
        """
        mb = self._items.get(item)
        if mb is None:
            mn = module_name(self.ref)
            # TODO: Revisit in dazl 7.0.0 when the internal structures of ModuleRef and DottedName change
            mb = ModuleHierarchy(
                package_ref(self.ref), (f"{mn}.{item}" if mn else str(mn)).split(".")
            )
            self._items[item] = mb
        return mb

    def add_module(self, module: "Module") -> None:
        obj = self
        components = str(module.name).split(".")
        for component in components:
            obj = obj[component]
        obj._modules.append(module)

    def __iter__(self):
        yield ModuleHierarchy.START, self.ref, None
        for child in self._items.values():
            yield from child
        for mod in self._modules:
            yield ModuleHierarchy.ITEM, self.ref, mod
        yield ModuleHierarchy.END, self.ref, None

    # def lines(self, depth: int = 0) -> Generator[str, None, None]:
    #     prefix = (depth * 4) * ' '
    #     for key, mb in self._items.items():
    #         yield ''
    #         yield ''
    #         yield f'{prefix}class {key}:'
    #         for leaf, template in mb._leaves.items():
    #             template_fn = template.data_type.name.full_name
    #             slot_names = tuple(name for name, _ in template.data_type.named_args)
    #
    #             yield f'{prefix}    class {leaf}(metaclass=TemplateMeta, template_name={template_fn!r}):'
    #             yield f'{prefix}        """'
    #             yield f'{prefix}        Example usage:'
    #             yield f'{prefix}            create({template_fn},'
    #             yield f'{prefix}                {python_example_object(self.store, template.data_type)}'
    #             yield f'{prefix}            )'
    #             for choice in template.choices:
    #                 yield f'{prefix}            exercise(cid, {choice.name!r}, {python_example_object(self.store, choice.data_type)})'
    #             yield f'{prefix}        """'
    #             yield f'{prefix}        __slots__ = {slot_names!r}'
    #             yield f'{prefix}'
    #             if slot_names:
    #                 yield f'{prefix}        def __init__(self, {", ".join(slot_names)}):'
    #                 for slot_name in slot_names:
    #                     yield f'{prefix}            self.{slot_name} = {slot_name}'
    #                 yield ''
    #                 yield f'{prefix}        def _asdict(self) -> dict:'
    #                 yield f'{prefix}            return {{'
    #                 for slot_name in slot_names:
    #                     yield f'{prefix}                {slot_name!r}: self.{slot_name},'
    #                 yield f'{prefix}                }}'
    #             else:
    #                 yield f'{prefix}        def _asdict(self) -> dict:'
    #                 yield f'{prefix}            return {{}}'
    #
    #             for choice in template.choices:
    #                 choice_data_type = self.store.resolve_type_reference(choice.data_type) if isinstance(choice.data_type, TypeReference) else choice.data_type
    #                 choice_slot_names = tuple(name for name, _ in choice_data_type.named_args) if isinstance(choice_data_type, RecordType) else ()
    #                 yield ''
    #                 yield f'{prefix}        class {choice.name}(metaclass=ChoiceMeta, template_name={template_fn!r}, choice_name={choice.name!r}):'
    #                 yield f'{prefix}            __slots__ = {choice_slot_names!r}'
    #
    #                 if choice_slot_names:
    #                     yield ''
    #                     yield f'{prefix}            def __init__(self, {", ".join(choice_slot_names)}):'
    #                     for choice_slot_name in choice_slot_names:
    #                         yield f'{prefix}                self.{choice_slot_name} = {choice_slot_name}'
    #                     yield ''
    #                     yield f'{prefix}            def _asdict(self) -> dict:'
    #                     yield f'{prefix}                return {{'
    #                     for choice_slot_name in choice_slot_names:
    #                         yield f'{prefix}                    {choice_slot_name!r}: self.{choice_slot_name},'
    #                     yield f'{prefix}                    }}'
    #                 else:
    #                     yield f'{prefix}            def _asdict(self) -> dict:'
    #                     yield f'{prefix}                return {{}}'
    #             yield f''
    #         for line in mb.lines(depth + 1):
    #             yield line

    def __repr__(self):
        return f"ModuleBuilder(ref={self.ref}, items={self._items})"



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module has been relocated to ``dazl.client``, ``dazl.damlast``, ``dazl.protocols``, or
``dazl.query``.
"""
from typing import TypeVar
import warnings

from ..client.errors import ConfigurationError, DazlPartyMissingError, UnknownTemplateWarning
from ..client.state import (
    ContractContextualData,
    ContractContextualDataCollection,
    ContractsHistoricalState,
    ContractsState,
)
from ..damlast.pkgfile import Dar
from ..prim import ContractData, ContractId, DazlError, DazlWarning, Party
from ..prim.errors import DazlImportError
from ..protocols.errors import ConnectionTimeoutError, UserTerminateRequest
from ..query import ContractMatch
from ..util.proc_util import ProcessDiedException

T = TypeVar("T")


__all__ = [
    "ConfigurationError",
    "ConnectionTimeoutError",
    "ContractContextualData",
    "ContractContextualDataCollection",
    "ContractData",
    "ContractId",
    "ContractMatch",
    "ContractsHistoricalState",
    "ContractsState",
    "Dar",
    "DazlError",
    "DazlImportError",
    "DazlPartyMissingError",
    "DazlWarning",
    "Party",
    "ProcessDiedException",
    "UnknownTemplateWarning",
    "UserTerminateRequest",
]


class CommandTimeoutError(DazlError):
    """
    Raised when a corresponding event for a command was not seen in the appropriate time window.
    """

    def __init__(self):
        warnings.warn(
            "This error is never raised; this symbol will be removed in dazl v9",
            DeprecationWarning,
            stacklevel=2,
        )


class ConnectionClosedError(DazlError):
    """
    Raised when trying to do something that requires a connection after connection pools have been
    closed.
    """

    def __init__(self):
        warnings.warn(
            "This error is never raised; this symbol will be removed in dazl v9",
            DeprecationWarning,
            stacklevel=2,
        )



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/trace_context.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/trace_context.proto",
    package="com.daml.ledger.api.v1",
    syntax="proto3",
    serialized_options=b"\n\026com.daml.ledger.api.v1B\026TraceContextOuterClass\252\002\026Com.Daml.Ledger.Api.V1",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n*com/daml/ledger/api/v1/trace_context.proto\x12\x16\x63om.daml.ledger.api.v1\x1a\x1egoogle/protobuf/wrappers.proto"\x9b\x01\n\x0cTraceContext\x12\x19\n\rtrace_id_high\x18\x01 \x01(\x04\x42\x02\x30\x01\x12\x14\n\x08trace_id\x18\x02 \x01(\x04\x42\x02\x30\x01\x12\x13\n\x07span_id\x18\x03 \x01(\x04\x42\x02\x30\x01\x12\x34\n\x0eparent_span_id\x18\x04 \x01(\x0b\x32\x1c.google.protobuf.UInt64Value\x12\x0f\n\x07sampled\x18\x05 \x01(\x08\x42I\n\x16\x63om.daml.ledger.api.v1B\x16TraceContextOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3',
    dependencies=[
        google_dot_protobuf_dot_wrappers__pb2.DESCRIPTOR,
    ],
)


_TRACECONTEXT = _descriptor.Descriptor(
    name="TraceContext",
    full_name="com.daml.ledger.api.v1.TraceContext",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="trace_id_high",
            full_name="com.daml.ledger.api.v1.TraceContext.trace_id_high",
            index=0,
            number=1,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=b"0\001",
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_id",
            full_name="com.daml.ledger.api.v1.TraceContext.trace_id",
            index=1,
            number=2,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=b"0\001",
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="span_id",
            full_name="com.daml.ledger.api.v1.TraceContext.span_id",
            index=2,
            number=3,
            type=4,
            cpp_type=4,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=b"0\001",
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="parent_span_id",
            full_name="com.daml.ledger.api.v1.TraceContext.parent_span_id",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="sampled",
            full_name="com.daml.ledger.api.v1.TraceContext.sampled",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=103,
    serialized_end=258,
)

_TRACECONTEXT.fields_by_name[
    "parent_span_id"
].message_type = google_dot_protobuf_dot_wrappers__pb2._UINT64VALUE
DESCRIPTOR.message_types_by_name["TraceContext"] = _TRACECONTEXT
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

TraceContext = _reflection.GeneratedProtocolMessageType(
    "TraceContext",
    (_message.Message,),
    {
        "DESCRIPTOR": _TRACECONTEXT,
        "__module__": "com.daml.ledger.api.v1.trace_context_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.TraceContext)
    },
)
_sym_db.RegisterMessage(TraceContext)


DESCRIPTOR._options = None
_TRACECONTEXT.fields_by_name["trace_id_high"]._options = None
_TRACECONTEXT.fields_by_name["trace_id"]._options = None
_TRACECONTEXT.fields_by_name["span_id"]._options = None
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/transaction_service.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import (
    ledger_offset_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2,
    trace_context_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2,
    transaction_filter_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__filter__pb2,
    transaction_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2,
)

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/transaction_service.proto",
    package="com.daml.ledger.api.v1",
    syntax="proto3",
    serialized_options=b"\n\026com.daml.ledger.api.v1B\034TransactionServiceOuterClass\252\002\026Com.Daml.Ledger.Api.V1",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n0com/daml/ledger/api/v1/transaction_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a*com/daml/ledger/api/v1/ledger_offset.proto\x1a/com/daml/ledger/api/v1/transaction_filter.proto\x1a(com/daml/ledger/api/v1/transaction.proto\x1a*com/daml/ledger/api/v1/trace_context.proto"\x9d\x02\n\x16GetTransactionsRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x33\n\x05\x62\x65gin\x18\x02 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffset\x12\x31\n\x03\x65nd\x18\x03 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffset\x12\x39\n\x06\x66ilter\x18\x04 \x01(\x0b\x32).com.daml.ledger.api.v1.TransactionFilter\x12\x0f\n\x07verbose\x18\x05 \x01(\x08\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext"T\n\x17GetTransactionsResponse\x12\x39\n\x0ctransactions\x18\x01 \x03(\x0b\x32#.com.daml.ledger.api.v1.Transaction"\\\n\x1bGetTransactionTreesResponse\x12=\n\x0ctransactions\x18\x01 \x03(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTree"\x9f\x01\n\x1eGetTransactionByEventIdRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x10\n\x08\x65vent_id\x18\x02 \x01(\t\x12\x1a\n\x12requesting_parties\x18\x03 \x03(\t\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext"\xa0\x01\n\x19GetTransactionByIdRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12\x16\n\x0etransaction_id\x18\x02 \x01(\t\x12\x1a\n\x12requesting_parties\x18\x03 \x03(\t\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext"V\n\x16GetTransactionResponse\x12<\n\x0btransaction\x18\x01 \x01(\x0b\x32\'.com.daml.ledger.api.v1.TransactionTree"V\n\x1aGetFlatTransactionResponse\x12\x38\n\x0btransaction\x18\x01 \x01(\x0b\x32#.com.daml.ledger.api.v1.Transaction"f\n\x13GetLedgerEndRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext"L\n\x14GetLedgerEndResponse\x12\x34\n\x06offset\x18\x01 \x01(\x0b\x32$.com.daml.ledger.api.v1.LedgerOffset2\xfd\x06\n\x12TransactionService\x12t\n\x0fGetTransactions\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a/.com.daml.ledger.api.v1.GetTransactionsResponse0\x01\x12|\n\x13GetTransactionTrees\x12..com.daml.ledger.api.v1.GetTransactionsRequest\x1a\x33.com.daml.ledger.api.v1.GetTransactionTreesResponse0\x01\x12\x81\x01\n\x17GetTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12w\n\x12GetTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a..com.daml.ledger.api.v1.GetTransactionResponse\x12\x89\x01\n\x1bGetFlatTransactionByEventId\x12\x36.com.daml.ledger.api.v1.GetTransactionByEventIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12\x7f\n\x16GetFlatTransactionById\x12\x31.com.daml.ledger.api.v1.GetTransactionByIdRequest\x1a\x32.com.daml.ledger.api.v1.GetFlatTransactionResponse\x12i\n\x0cGetLedgerEnd\x12+.com.daml.ledger.api.v1.GetLedgerEndRequest\x1a,.com.daml.ledger.api.v1.GetLedgerEndResponseBO\n\x16\x63om.daml.ledger.api.v1B\x1cTransactionServiceOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3',
    dependencies=[
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2.DESCRIPTOR,
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__filter__pb2.DESCRIPTOR,
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2.DESCRIPTOR,
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2.DESCRIPTOR,
    ],
)


_GETTRANSACTIONSREQUEST = _descriptor.Descriptor(
    name="GetTransactionsRequest",
    full_name="com.daml.ledger.api.v1.GetTransactionsRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.GetTransactionsRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="begin",
            full_name="com.daml.ledger.api.v1.GetTransactionsRequest.begin",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="end",
            full_name="com.daml.ledger.api.v1.GetTransactionsRequest.end",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="filter",
            full_name="com.daml.ledger.api.v1.GetTransactionsRequest.filter",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="verbose",
            full_name="com.daml.ledger.api.v1.GetTransactionsRequest.verbose",
            index=4,
            number=5,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.GetTransactionsRequest.trace_context",
            index=5,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=256,
    serialized_end=541,
)


_GETTRANSACTIONSRESPONSE = _descriptor.Descriptor(
    name="GetTransactionsResponse",
    full_name="com.daml.ledger.api.v1.GetTransactionsResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="transactions",
            full_name="com.daml.ledger.api.v1.GetTransactionsResponse.transactions",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=543,
    serialized_end=627,
)


_GETTRANSACTIONTREESRESPONSE = _descriptor.Descriptor(
    name="GetTransactionTreesResponse",
    full_name="com.daml.ledger.api.v1.GetTransactionTreesResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="transactions",
            full_name="com.daml.ledger.api.v1.GetTransactionTreesResponse.transactions",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=629,
    serialized_end=721,
)


_GETTRANSACTIONBYEVENTIDREQUEST = _descriptor.Descriptor(
    name="GetTransactionByEventIdRequest",
    full_name="com.daml.ledger.api.v1.GetTransactionByEventIdRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.GetTransactionByEventIdRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="event_id",
            full_name="com.daml.ledger.api.v1.GetTransactionByEventIdRequest.event_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="requesting_parties",
            full_name="com.daml.ledger.api.v1.GetTransactionByEventIdRequest.requesting_parties",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.GetTransactionByEventIdRequest.trace_context",
            index=3,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=724,
    serialized_end=883,
)


_GETTRANSACTIONBYIDREQUEST = _descriptor.Descriptor(
    name="GetTransactionByIdRequest",
    full_name="com.daml.ledger.api.v1.GetTransactionByIdRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.GetTransactionByIdRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="transaction_id",
            full_name="com.daml.ledger.api.v1.GetTransactionByIdRequest.transaction_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="requesting_parties",
            full_name="com.daml.ledger.api.v1.GetTransactionByIdRequest.requesting_parties",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.GetTransactionByIdRequest.trace_context",
            index=3,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=886,
    serialized_end=1046,
)


_GETTRANSACTIONRESPONSE = _descriptor.Descriptor(
    name="GetTransactionResponse",
    full_name="com.daml.ledger.api.v1.GetTransactionResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="transaction",
            full_name="com.daml.ledger.api.v1.GetTransactionResponse.transaction",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1048,
    serialized_end=1134,
)


_GETFLATTRANSACTIONRESPONSE = _descriptor.Descriptor(
    name="GetFlatTransactionResponse",
    full_name="com.daml.ledger.api.v1.GetFlatTransactionResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="transaction",
            full_name="com.daml.ledger.api.v1.GetFlatTransactionResponse.transaction",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1136,
    serialized_end=1222,
)


_GETLEDGERENDREQUEST = _descriptor.Descriptor(
    name="GetLedgerEndRequest",
    full_name="com.daml.ledger.api.v1.GetLedgerEndRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.GetLedgerEndRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.GetLedgerEndRequest.trace_context",
            index=1,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1224,
    serialized_end=1326,
)


_GETLEDGERENDRESPONSE = _descriptor.Descriptor(
    name="GetLedgerEndResponse",
    full_name="com.daml.ledger.api.v1.GetLedgerEndResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="offset",
            full_name="com.daml.ledger.api.v1.GetLedgerEndResponse.offset",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1328,
    serialized_end=1404,
)

_GETTRANSACTIONSREQUEST.fields_by_name[
    "begin"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2._LEDGEROFFSET
_GETTRANSACTIONSREQUEST.fields_by_name[
    "end"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2._LEDGEROFFSET
_GETTRANSACTIONSREQUEST.fields_by_name[
    "filter"
].message_type = (
    com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__filter__pb2._TRANSACTIONFILTER
)
_GETTRANSACTIONSREQUEST.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_GETTRANSACTIONSRESPONSE.fields_by_name[
    "transactions"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2._TRANSACTION
_GETTRANSACTIONTREESRESPONSE.fields_by_name[
    "transactions"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2._TRANSACTIONTREE
_GETTRANSACTIONBYEVENTIDREQUEST.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_GETTRANSACTIONBYIDREQUEST.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_GETTRANSACTIONRESPONSE.fields_by_name[
    "transaction"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2._TRANSACTIONTREE
_GETFLATTRANSACTIONRESPONSE.fields_by_name[
    "transaction"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_transaction__pb2._TRANSACTION
_GETLEDGERENDREQUEST.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
_GETLEDGERENDRESPONSE.fields_by_name[
    "offset"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_ledger__offset__pb2._LEDGEROFFSET
DESCRIPTOR.message_types_by_name["GetTransactionsRequest"] = _GETTRANSACTIONSREQUEST
DESCRIPTOR.message_types_by_name["GetTransactionsResponse"] = _GETTRANSACTIONSRESPONSE
DESCRIPTOR.message_types_by_name["GetTransactionTreesResponse"] = _GETTRANSACTIONTREESRESPONSE
DESCRIPTOR.message_types_by_name["GetTransactionByEventIdRequest"] = _GETTRANSACTIONBYEVENTIDREQUEST
DESCRIPTOR.message_types_by_name["GetTransactionByIdRequest"] = _GETTRANSACTIONBYIDREQUEST
DESCRIPTOR.message_types_by_name["GetTransactionResponse"] = _GETTRANSACTIONRESPONSE
DESCRIPTOR.message_types_by_name["GetFlatTransactionResponse"] = _GETFLATTRANSACTIONRESPONSE
DESCRIPTOR.message_types_by_name["GetLedgerEndRequest"] = _GETLEDGERENDREQUEST
DESCRIPTOR.message_types_by_name["GetLedgerEndResponse"] = _GETLEDGERENDRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetTransactionsRequest = _reflection.GeneratedProtocolMessageType(
    "GetTransactionsRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTRANSACTIONSREQUEST,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionsRequest)
    },
)
_sym_db.RegisterMessage(GetTransactionsRequest)

GetTransactionsResponse = _reflection.GeneratedProtocolMessageType(
    "GetTransactionsResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTRANSACTIONSRESPONSE,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionsResponse)
    },
)
_sym_db.RegisterMessage(GetTransactionsResponse)

GetTransactionTreesResponse = _reflection.GeneratedProtocolMessageType(
    "GetTransactionTreesResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTRANSACTIONTREESRESPONSE,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionTreesResponse)
    },
)
_sym_db.RegisterMessage(GetTransactionTreesResponse)

GetTransactionByEventIdRequest = _reflection.GeneratedProtocolMessageType(
    "GetTransactionByEventIdRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTRANSACTIONBYEVENTIDREQUEST,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionByEventIdRequest)
    },
)
_sym_db.RegisterMessage(GetTransactionByEventIdRequest)

GetTransactionByIdRequest = _reflection.GeneratedProtocolMessageType(
    "GetTransactionByIdRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTRANSACTIONBYIDREQUEST,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionByIdRequest)
    },
)
_sym_db.RegisterMessage(GetTransactionByIdRequest)

GetTransactionResponse = _reflection.GeneratedProtocolMessageType(
    "GetTransactionResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTRANSACTIONRESPONSE,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetTransactionResponse)
    },
)
_sym_db.RegisterMessage(GetTransactionResponse)

GetFlatTransactionResponse = _reflection.GeneratedProtocolMessageType(
    "GetFlatTransactionResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETFLATTRANSACTIONRESPONSE,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetFlatTransactionResponse)
    },
)
_sym_db.RegisterMessage(GetFlatTransactionResponse)

GetLedgerEndRequest = _reflection.GeneratedProtocolMessageType(
    "GetLedgerEndRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETLEDGERENDREQUEST,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerEndRequest)
    },
)
_sym_db.RegisterMessage(GetLedgerEndRequest)

GetLedgerEndResponse = _reflection.GeneratedProtocolMessageType(
    "GetLedgerEndResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETLEDGERENDRESPONSE,
        "__module__": "com.daml.ledger.api.v1.transaction_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerEndResponse)
    },
)
_sym_db.RegisterMessage(GetLedgerEndResponse)


DESCRIPTOR._options = None

_TRANSACTIONSERVICE = _descriptor.ServiceDescriptor(
    name="TransactionService",
    full_name="com.daml.ledger.api.v1.TransactionService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=1407,
    serialized_end=2300,
    methods=[
        _descriptor.MethodDescriptor(
            name="GetTransactions",
            full_name="com.daml.ledger.api.v1.TransactionService.GetTransactions",
            index=0,
            containing_service=None,
            input_type=_GETTRANSACTIONSREQUEST,
            output_type=_GETTRANSACTIONSRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetTransactionTrees",
            full_name="com.daml.ledger.api.v1.TransactionService.GetTransactionTrees",
            index=1,
            containing_service=None,
            input_type=_GETTRANSACTIONSREQUEST,
            output_type=_GETTRANSACTIONTREESRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetTransactionByEventId",
            full_name="com.daml.ledger.api.v1.TransactionService.GetTransactionByEventId",
            index=2,
            containing_service=None,
            input_type=_GETTRANSACTIONBYEVENTIDREQUEST,
            output_type=_GETTRANSACTIONRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetTransactionById",
            full_name="com.daml.ledger.api.v1.TransactionService.GetTransactionById",
            index=3,
            containing_service=None,
            input_type=_GETTRANSACTIONBYIDREQUEST,
            output_type=_GETTRANSACTIONRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetFlatTransactionByEventId",
            full_name="com.daml.ledger.api.v1.TransactionService.GetFlatTransactionByEventId",
            index=4,
            containing_service=None,
            input_type=_GETTRANSACTIONBYEVENTIDREQUEST,
            output_type=_GETFLATTRANSACTIONRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetFlatTransactionById",
            full_name="com.daml.ledger.api.v1.TransactionService.GetFlatTransactionById",
            index=5,
            containing_service=None,
            input_type=_GETTRANSACTIONBYIDREQUEST,
            output_type=_GETFLATTRANSACTIONRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="GetLedgerEnd",
            full_name="com.daml.ledger.api.v1.TransactionService.GetLedgerEnd",
            index=6,
            containing_service=None,
            input_type=_GETLEDGERENDREQUEST,
            output_type=_GETLEDGERENDRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_TRANSACTIONSERVICE)

DESCRIPTOR.services_by_name["TransactionService"] = _TRANSACTIONSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
from threading import Event

from ..scheduler import RunLevel


class RunState:
    """
    Manage the current run level of the client.
    """

    def __init__(self, level=RunLevel.RUN_FOREVER):
        self._terminate = Event()
        self.level = level

    def handle_sigint(self):
        self.level = RunLevel.TERMINATE_GRACEFULLY
        self._terminate.set()

    def handle_sigquit(self):
        self.level = RunLevel.TERMINATE_IMMEDIATELY
        self._terminate.set()

    @property
    def terminate_requested(self) -> bool:
        """
        Indication of whether the user has requested a shutdown.
        """
        return self._terminate.is_set()



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/admin/config_management_service.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import (
    duration_pb2 as google_dot_protobuf_dot_duration__pb2,
    timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2,
)

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/admin/config_management_service.proto",
    package="com.daml.ledger.api.v1.admin",
    syntax="proto3",
    serialized_options=b"\n\034com.daml.ledger.api.v1.adminB!ConfigManagementServiceOuterClass\252\002\034Com.Daml.Ledger.Api.V1.Admin",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n<com/daml/ledger/api/v1/admin/config_management_service.proto\x12\x1c\x63om.daml.ledger.api.v1.admin\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x15\n\x13GetTimeModelRequest"u\n\x14GetTimeModelResponse\x12 \n\x18\x63onfiguration_generation\x18\x01 \x01(\x03\x12;\n\ntime_model\x18\x02 \x01(\x0b\x32\'.com.daml.ledger.api.v1.admin.TimeModel"\xc8\x01\n\x13SetTimeModelRequest\x12\x15\n\rsubmission_id\x18\x01 \x01(\t\x12\x37\n\x13maximum_record_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12 \n\x18\x63onfiguration_generation\x18\x03 \x01(\x03\x12?\n\x0enew_time_model\x18\x04 \x01(\x0b\x32\'.com.daml.ledger.api.v1.admin.TimeModel"8\n\x14SetTimeModelResponse\x12 \n\x18\x63onfiguration_generation\x18\x01 \x01(\x03"\xb3\x01\n\tTimeModel\x12:\n\x17\x61vg_transaction_latency\x18\x04 \x01(\x0b\x32\x19.google.protobuf.Duration\x12+\n\x08min_skew\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12+\n\x08max_skew\x18\x06 \x01(\x0b\x32\x19.google.protobuf.DurationJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04\x32\x87\x02\n\x17\x43onfigManagementService\x12u\n\x0cGetTimeModel\x12\x31.com.daml.ledger.api.v1.admin.GetTimeModelRequest\x1a\x32.com.daml.ledger.api.v1.admin.GetTimeModelResponse\x12u\n\x0cSetTimeModel\x12\x31.com.daml.ledger.api.v1.admin.SetTimeModelRequest\x1a\x32.com.daml.ledger.api.v1.admin.SetTimeModelResponseB`\n\x1c\x63om.daml.ledger.api.v1.adminB!ConfigManagementServiceOuterClass\xaa\x02\x1c\x43om.Daml.Ledger.Api.V1.Adminb\x06proto3',
    dependencies=[
        google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,
    ],
)


_GETTIMEMODELREQUEST = _descriptor.Descriptor(
    name="GetTimeModelRequest",
    full_name="com.daml.ledger.api.v1.admin.GetTimeModelRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=159,
    serialized_end=180,
)


_GETTIMEMODELRESPONSE = _descriptor.Descriptor(
    name="GetTimeModelResponse",
    full_name="com.daml.ledger.api.v1.admin.GetTimeModelResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="configuration_generation",
            full_name="com.daml.ledger.api.v1.admin.GetTimeModelResponse.configuration_generation",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="time_model",
            full_name="com.daml.ledger.api.v1.admin.GetTimeModelResponse.time_model",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=182,
    serialized_end=299,
)


_SETTIMEMODELREQUEST = _descriptor.Descriptor(
    name="SetTimeModelRequest",
    full_name="com.daml.ledger.api.v1.admin.SetTimeModelRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="submission_id",
            full_name="com.daml.ledger.api.v1.admin.SetTimeModelRequest.submission_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="maximum_record_time",
            full_name="com.daml.ledger.api.v1.admin.SetTimeModelRequest.maximum_record_time",
            index=1,
            number=2,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="configuration_generation",
            full_name="com.daml.ledger.api.v1.admin.SetTimeModelRequest.configuration_generation",
            index=2,
            number=3,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="new_time_model",
            full_name="com.daml.ledger.api.v1.admin.SetTimeModelRequest.new_time_model",
            index=3,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=302,
    serialized_end=502,
)


_SETTIMEMODELRESPONSE = _descriptor.Descriptor(
    name="SetTimeModelResponse",
    full_name="com.daml.ledger.api.v1.admin.SetTimeModelResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="configuration_generation",
            full_name="com.daml.ledger.api.v1.admin.SetTimeModelResponse.configuration_generation",
            index=0,
            number=1,
            type=3,
            cpp_type=2,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=504,
    serialized_end=560,
)


_TIMEMODEL = _descriptor.Descriptor(
    name="TimeModel",
    full_name="com.daml.ledger.api.v1.admin.TimeModel",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="avg_transaction_latency",
            full_name="com.daml.ledger.api.v1.admin.TimeModel.avg_transaction_latency",
            index=0,
            number=4,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="min_skew",
            full_name="com.daml.ledger.api.v1.admin.TimeModel.min_skew",
            index=1,
            number=5,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="max_skew",
            full_name="com.daml.ledger.api.v1.admin.TimeModel.max_skew",
            index=2,
            number=6,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=563,
    serialized_end=742,
)

_GETTIMEMODELRESPONSE.fields_by_name["time_model"].message_type = _TIMEMODEL
_SETTIMEMODELREQUEST.fields_by_name[
    "maximum_record_time"
].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_SETTIMEMODELREQUEST.fields_by_name["new_time_model"].message_type = _TIMEMODEL
_TIMEMODEL.fields_by_name[
    "avg_transaction_latency"
].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_TIMEMODEL.fields_by_name["min_skew"].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_TIMEMODEL.fields_by_name["max_skew"].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
DESCRIPTOR.message_types_by_name["GetTimeModelRequest"] = _GETTIMEMODELREQUEST
DESCRIPTOR.message_types_by_name["GetTimeModelResponse"] = _GETTIMEMODELRESPONSE
DESCRIPTOR.message_types_by_name["SetTimeModelRequest"] = _SETTIMEMODELREQUEST
DESCRIPTOR.message_types_by_name["SetTimeModelResponse"] = _SETTIMEMODELRESPONSE
DESCRIPTOR.message_types_by_name["TimeModel"] = _TIMEMODEL
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetTimeModelRequest = _reflection.GeneratedProtocolMessageType(
    "GetTimeModelRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTIMEMODELREQUEST,
        "__module__": "com.daml.ledger.api.v1.admin.config_management_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetTimeModelRequest)
    },
)
_sym_db.RegisterMessage(GetTimeModelRequest)

GetTimeModelResponse = _reflection.GeneratedProtocolMessageType(
    "GetTimeModelResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETTIMEMODELRESPONSE,
        "__module__": "com.daml.ledger.api.v1.admin.config_management_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.GetTimeModelResponse)
    },
)
_sym_db.RegisterMessage(GetTimeModelResponse)

SetTimeModelRequest = _reflection.GeneratedProtocolMessageType(
    "SetTimeModelRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _SETTIMEMODELREQUEST,
        "__module__": "com.daml.ledger.api.v1.admin.config_management_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.SetTimeModelRequest)
    },
)
_sym_db.RegisterMessage(SetTimeModelRequest)

SetTimeModelResponse = _reflection.GeneratedProtocolMessageType(
    "SetTimeModelResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _SETTIMEMODELRESPONSE,
        "__module__": "com.daml.ledger.api.v1.admin.config_management_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.SetTimeModelResponse)
    },
)
_sym_db.RegisterMessage(SetTimeModelResponse)

TimeModel = _reflection.GeneratedProtocolMessageType(
    "TimeModel",
    (_message.Message,),
    {
        "DESCRIPTOR": _TIMEMODEL,
        "__module__": "com.daml.ledger.api.v1.admin.config_management_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.admin.TimeModel)
    },
)
_sym_db.RegisterMessage(TimeModel)


DESCRIPTOR._options = None

_CONFIGMANAGEMENTSERVICE = _descriptor.ServiceDescriptor(
    name="ConfigManagementService",
    full_name="com.daml.ledger.api.v1.admin.ConfigManagementService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=745,
    serialized_end=1008,
    methods=[
        _descriptor.MethodDescriptor(
            name="GetTimeModel",
            full_name="com.daml.ledger.api.v1.admin.ConfigManagementService.GetTimeModel",
            index=0,
            containing_service=None,
            input_type=_GETTIMEMODELREQUEST,
            output_type=_GETTIMEMODELRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="SetTimeModel",
            full_name="com.daml.ledger.api.v1.admin.ConfigManagementService.SetTimeModel",
            index=1,
            containing_service=None,
            input_type=_SETTIMEMODELREQUEST,
            output_type=_SETTIMEMODELRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_CONFIGMANAGEMENTSERVICE)

DESCRIPTOR.services_by_name["ConfigManagementService"] = _CONFIGMANAGEMENTSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
DAML-LF fast lookups
--------------------
"""

# Implementation notes:
#
# The code in here is fairly monotonous and boilerplate heavy. However, being too creative here can
# potentially lead to performance degradations, particularly at application startup where type and
# template lookups by name are very frequent. Please be conscious of the runtime costs of
# modifications in this file!

import threading
from types import MappingProxyType
from typing import AbstractSet, Any, Collection, Dict, Iterable, NoReturn, Optional, Tuple

from .daml_lf_1 import (
    Archive,
    DefDataType,
    DefTemplate,
    DefValue,
    DottedName,
    ModuleRef,
    Package,
    PackageRef,
    TemplateChoice,
    TypeConName,
    ValName,
)
from .errors import NameNotFoundError, PackageNotFoundError
from .protocols import SymbolLookup

__all__ = [
    "find_choice",
    "parse_type_con_name",
    "validate_template",
    "EmptyLookup",
    "MultiPackageLookup",
    "PackageLookup",
    "MultiPackageLookup",
]

STAR = PackageRef("*")


def parse_type_con_name(val: str) -> "TypeConName":
    """
    Parse the given string as a type constructor.
    """
    pkg, name = validate_template(val)
    module_name, _, entity_name = name.rpartition(":")
    module_ref = ModuleRef(pkg, DottedName(module_name.split(".")))
    return TypeConName(module_ref, entity_name.split("."))


def empty_lookup_impl(ref: "Any") -> "NoReturn":
    pkg, _ = validate_template(ref)
    if pkg != STAR:
        raise PackageNotFoundError(pkg)
    else:
        raise NameNotFoundError(ref)


def validate_template(template: "Any") -> "Tuple[PackageRef, str]":
    """
    Return a module and type name component from something that can be interpreted as a template.

    :param template:
        Any object that can be interpreted as an identifier for a template.
    :return:
        A tuple of package ID and ``Module.Name:EntityName`` (the package-scoped identifier for the
        type). The special value ``'*'`` is used if either the package ID, module name, or both
        should be wildcarded.
    :raise ValueError:
        If the object could not be interpreted as a thing referring to a template.
    """
    from ..damlast.daml_lf_1 import TypeConName
    from ..damlast.util import package_local_name, package_ref

    if template == "*" or template is None:
        return STAR, "*"

    if isinstance(template, str):
        components = template.split(":")
        if len(components) == 3:
            # correct number of colons for a fully-qualified name
            pkgid, m, e = components
            return pkgid, f"{m}:{e}"

        elif len(components) == 2:
            # one colon, so assume the package ID is unspecified UNLESS the second component is a
            # wildcard; then we assume the wildcard means any module name and entity name
            m, e = components
            return (STAR, f"{m}:{e}") if e != "*" else (PackageRef(m), "*")

        else:
            raise ValueError("string must be in the format PKG_REF:MOD:ENTITY or MOD:ENTITY")

    if isinstance(template, TypeConName):
        return package_ref(template), package_local_name(template)
    else:
        raise ValueError(f"Don't know how to convert {template!r} into a template")


class EmptyLookup(SymbolLookup):
    """
    A :class:`SymbolLookup` that trivially throws for all of its functions.

    This can be used where a :class:`SymbolLookup` instance is useful but an implementation is not
    required.

    All methods are implemented such that if the provided ref has a package ID,
    :class:`PackageNotFoundError` is thrown; otherwise, :class:`NameNotFoundError` is thrown.
    """

    __slots__ = ()

    def archives(self) -> "Collection[Archive]":
        return frozenset()

    def package_ids(self) -> "AbstractSet[PackageRef]":
        return frozenset()

    def data_type_name(self, ref: "Any") -> "NoReturn":
        return empty_lookup_impl(ref)

    def data_type(self, ref: "Any") -> "NoReturn":
        return empty_lookup_impl(ref)

    def value(self, ref: "Any") -> "NoReturn":
        return empty_lookup_impl(ref)

    def template_names(self, ref: "Any") -> "Collection[TypeConName]":
        return frozenset()

    def template_name(self, ref: "Any") -> "NoReturn":
        return empty_lookup_impl(ref)

    def template(self, ref: "Any") -> "NoReturn":
        return empty_lookup_impl(ref)


class PackageLookup(SymbolLookup):
    """
    Caching structure to make lookups on type names within a :class:`Package` faster.
    """

    def __init__(self, archive: "Archive"):
        self.archive = archive

        data_types = {}  # type: Dict[str, Tuple[TypeConName, DefDataType]]
        values = {}  # type: Dict[str, Tuple[ValName, DefValue]]
        templates = {}  # type: Dict[str, Tuple[TypeConName, DefTemplate]]
        for module in self.archive.package.modules:
            module_ref = ModuleRef(archive.hash, module.name)

            for dt in module.data_types:
                dt_name = TypeConName(module_ref, dt.name.segments)
                data_types[f"{module.name}:{dt.name}"] = (dt_name, dt)

            for value in module.values:
                value_name = ValName(module_ref, value.name_with_type.name)
                values[f"{module.name}:{value.name_with_type.name}"] = (value_name, value)

            for tmpl in module.templates:
                tmpl_name = TypeConName(module_ref, tmpl.tycon.segments)
                templates[f"{module.name}:{tmpl.tycon}"] = (tmpl_name, tmpl)

        self._data_types = MappingProxyType(data_types)
        self._values = MappingProxyType(values)
        self._templates = MappingProxyType(templates)

    def archives(self) -> "Collection[Archive]":
        return [self.archive]

    def package_ids(self) -> "AbstractSet[PackageRef]":
        return frozenset([self.archive.hash])

    def data_type_name(self, ref: "Any") -> "TypeConName":
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            dt_name = self.local_data_type_name(name)
            if dt_name is not None:
                return dt_name

        raise NameNotFoundError(ref)

    def data_type(self, ref: "Any") -> "DefDataType":
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            dt = self.local_data_type(name)
            if dt is not None:
                return dt

        raise NameNotFoundError(ref)

    def local_data_type_name(self, name: str) -> "Optional[TypeConName]":
        r = self._data_types.get(name)
        return r[0] if r is not None else None

    def local_data_type(self, name: str) -> "Optional[DefDataType]":
        """
        Variation of :meth:`data_type` that assumes the name is already scoped to this package.
        Unlike :meth:`data_type`, this method returns ``None`` in the case of no match.

        You should not normally use this method directly, and instead prefer to use the methods of
        the :class:`SymbolLookup` protocol.

        :param name:
            A name to search for. Must be of the form ``"ModuleName:EntityName"``, where both
            modules and entities are dot-delimited.
        :return:
            Either a matching :class:`DefDataType`, or ``None`` if no match.
        """
        r = self._data_types.get(name)
        return r[1] if r is not None else None

    def value(self, ref: "Any") -> "DefValue":
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            dt = self.local_value(name)
            if dt is not None:
                return dt

        raise NameNotFoundError(ref)

    def local_value(self, name: str) -> "Optional[DefValue]":
        """
        Variation of :meth:`data_type` that assumes the name is already scoped to this package.
        Unlike :meth:`data_type`, this method returns ``None`` in the case of no match.

        You should not normally use this method directly, and instead prefer to use the methods of
        the :class:`SymbolLookup` protocol.

        :param name:
            A name to search for. Must be of the form ``"ModuleName:EntityName"``, where both
            modules and entities are dot-delimited.
        :return:
            Either a matching :class:`DefDataType`, or ``None`` if no match.
        """
        r = self._values.get(name)
        return r[1] if r is not None else None

    def template_names(self, ref: "Any") -> "Collection[TypeConName]":
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            if name == "*":
                return self.local_template_names()
            elif name in self._templates:
                n, _ = self._templates.get(name)
                return n
        return []

    def template_name(self, ref: "Any") -> "TypeConName":
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            tmpl = self.local_template_name(name)
            if tmpl is not None:
                return tmpl

        raise NameNotFoundError(ref)

    def template(self, ref: "Any") -> "DefTemplate":
        pkg, name = validate_template(ref)
        if pkg == self.archive.hash or pkg == STAR:
            tmpl = self.local_template(name)
            if tmpl is not None:
                return tmpl

        raise NameNotFoundError(ref)

    def local_template_names(self) -> "Collection[TypeConName]":
        return [n for n, _ in self._templates.values()]

    def local_template_name(self, name: str) -> "Optional[TypeConName]":
        r = self._templates.get(name)
        return r[0] if r is not None else None

    def local_template(self, name: str) -> "Optional[DefTemplate]":
        """
        Variation of :meth:`data_type` that assumes the name is already scoped to this package.
        Unlike :meth:`data_type`, this method returns ``None`` in the case of no match.

        You should not normally use this method directly, and instead prefer to use the methods of
        the :class:`SymbolLookup` protocol.

        :param name:
            A name to search for. Must be of the form ``"ModuleName:EntityName"``, where both
            modules and entities are dot-delimited.
        :return:
            Either a matching :class:`DefDataType`, or ``None`` if no match.
        """
        r = self._templates.get(name)
        return r[1] if r is not None else None


class MultiPackageLookup(SymbolLookup):
    """
    Combines lookups across multiple archives.

    This class is thread-safe. When calling :meth:`add_archive` and any of the other read-only
    methods concurrently, read-only methods will NOT block; they will return the previous state
    of the lookup.

    Packages can only be added; they cannot be removed once added.
    """

    def __init__(self, archives: "Optional[Collection[Archive]]" = None):
        self._lock = threading.Lock()
        self._cache = {}  # type: Dict[PackageRef, PackageLookup]
        if archives is not None:
            self.add_archive(*archives)

    def archives(self) -> "Collection[Archive]":
        """
        Return the list of known archives.
        """
        return [lookup.archive for lookup in self._cache.values()]

    def add_archive(self, *a: "Archive") -> None:
        """
        Add one or more :class:`Archive`s to this lookup.

        This method is thread-safe, but note that :class:`MultiPackageLookup` allows dirty reads for
        performance reasons.

        :param a: One or more :class:`Archive`s to add.
        """
        new_lookups = {ar.hash: PackageLookup(ar) for ar in a}
        with self._lock:
            # replace the old cache with a new one, incorporating values from both the existing
            # cache and the new lookups that came in. When there is a key conflict between the two
            # APIs, always prefer the existing cache to provide stability to callers.
            self._cache = {**new_lookups, **self._cache}

    def package_ids(self) -> "AbstractSet[PackageRef]":
        return set(self._cache)

    def package(self, ref: "PackageRef") -> "Package":
        lookup = self._cache.get(ref)
        if lookup is not None:
            return lookup.archive.package

        raise PackageNotFoundError(ref)

    def data_type_name(self, ref: "Any") -> "TypeConName":
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            dt_name = lookup.local_data_type_name(name)
            if dt_name is not None:
                return dt_name

        raise NameNotFoundError(ref)

    def data_type(self, ref: "Any") -> "DefDataType":
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            dt = lookup.local_data_type(name)
            if dt is not None:
                return dt

        raise NameNotFoundError(ref)

    def value(self, ref: "Any") -> "DefValue":
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            val = lookup.local_value(name)
            if val is not None:
                return val

        raise NameNotFoundError(ref)

    def template_names(self, ref: "Any") -> "Collection[TypeConName]":
        names = []

        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            if name == "*":
                names.extend(lookup.local_template_names())
            else:
                n = lookup.local_template_name(name)
                if n is not None:
                    names.append(n)

        return names

    def template_name(self, ref: "Any") -> "TypeConName":
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            n = lookup.local_template_name(name)
            if n is not None:
                return n

        raise NameNotFoundError(ref)

    def template(self, ref: "Any") -> "DefTemplate":
        pkg, name = validate_template(ref)
        for lookup in self._lookups(pkg):
            tmpl = lookup.local_template(name)
            if tmpl is not None:
                return tmpl

        raise NameNotFoundError(ref)

    def _lookups(self, ref: "PackageRef") -> "Iterable[PackageLookup]":
        """
        Return the individual :class:`PackageLookup` objects that should be consulted based on the
        :class:`PackageRef`.

        :param ref:
            A :class:`PackageRef` to look for.
        :return:
            A collection of :class:`PackageLookup` objects that match the :class:`PackageRef`. This
            collection is never empty.
        :raises PackageNotFoundError:
            if the :class:`PackageRef` points to a package that is not present in this lookup.
        """
        if ref == "*":
            return self._cache.values()

        lookup = self._cache.get(ref)
        if lookup is not None:
            return (lookup,)

        raise PackageNotFoundError(ref)


def find_choice(template: "DefTemplate", name: str) -> "TemplateChoice":
    """
    Find a choice in a :class:`DefTemplate`. If the choice could not be found,
    :class:`NameNotFoundError` is raised.
    """
    for choice in template.choices:
        if choice.name == name:
            return choice

    raise NameNotFoundError(f"choice {name}")



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from ...._gen.com.daml.daml_lf_dev.daml_lf_1_pb2 import (
    DefDataType,
    DefTemplate,
    PrimType,
    TemplateChoice,
)
from ...._gen.com.daml.daml_lf_dev.daml_lf_pb2 import ArchivePayload
from ...._gen.com.daml.ledger.api.v1.active_contracts_service_pb2 import (
    GetActiveContractsRequest,
    GetActiveContractsResponse,
)
from ...._gen.com.daml.ledger.api.v1.active_contracts_service_pb2_grpc import (
    ActiveContractsServiceStub,
)
from ...._gen.com.daml.ledger.api.v1.admin.package_management_service_pb2 import (
    UploadDarFileRequest,
    UploadDarFileResponse,
)
from ...._gen.com.daml.ledger.api.v1.admin.package_management_service_pb2_grpc import (
    PackageManagementServiceStub,
)
from ...._gen.com.daml.ledger.api.v1.command_service_pb2_grpc import CommandServiceStub
from ...._gen.com.daml.ledger.api.v1.command_submission_service_pb2 import SubmitRequest
from ...._gen.com.daml.ledger.api.v1.commands_pb2 import (
    Command,
    Commands,
    CreateAndExerciseCommand,
    CreateCommand,
    ExerciseByKeyCommand,
    ExerciseCommand,
)
from ...._gen.com.daml.ledger.api.v1.event_pb2 import Event
from ...._gen.com.daml.ledger.api.v1.ledger_identity_service_pb2 import GetLedgerIdentityRequest
from ...._gen.com.daml.ledger.api.v1.ledger_identity_service_pb2_grpc import (
    LedgerIdentityServiceStub,
)
from ...._gen.com.daml.ledger.api.v1.ledger_offset_pb2 import LedgerOffset
from ...._gen.com.daml.ledger.api.v1.package_service_pb2 import (
    GetPackageRequest,
    ListPackagesRequest,
)
from ...._gen.com.daml.ledger.api.v1.package_service_pb2_grpc import PackageServiceStub
from ...._gen.com.daml.ledger.api.v1.testing.time_service_pb2 import GetTimeRequest, SetTimeRequest
from ...._gen.com.daml.ledger.api.v1.testing.time_service_pb2_grpc import TimeServiceStub
from ...._gen.com.daml.ledger.api.v1.transaction_filter_pb2 import (
    Filters,
    InclusiveFilters,
    TransactionFilter,
)
from ...._gen.com.daml.ledger.api.v1.transaction_pb2 import Transaction
from ...._gen.com.daml.ledger.api.v1.transaction_service_pb2 import (
    GetLedgerEndRequest,
    GetTransactionByEventIdRequest,
    GetTransactionsRequest,
    GetTransactionsResponse,
)
from ...._gen.com.daml.ledger.api.v1.transaction_service_pb2_grpc import TransactionServiceStub
from ...._gen.com.daml.ledger.api.v1.value_pb2 import (
    Identifier,
    List,
    Record,
    RecordField,
    Value,
    Variant,
)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import (
    config_management_service_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2,
)


class ConfigManagementServiceStub(object):
    """Status: experimental interface, will change before it is deemed production
    ready

    Ledger configuration management service provides methods for the ledger administrator
    to change the current ledger configuration. The services provides methods to modify
    different aspects of the configuration.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetTimeModel = channel.unary_unary(
            "/com.daml.ledger.api.v1.admin.ConfigManagementService/GetTimeModel",
            request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.GetTimeModelRequest.SerializeToString,
            response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.GetTimeModelResponse.FromString,
        )
        self.SetTimeModel = channel.unary_unary(
            "/com.daml.ledger.api.v1.admin.ConfigManagementService/SetTimeModel",
            request_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.SetTimeModelRequest.SerializeToString,
            response_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.SetTimeModelResponse.FromString,
        )


class ConfigManagementServiceServicer(object):
    """Status: experimental interface, will change before it is deemed production
    ready

    Ledger configuration management service provides methods for the ledger administrator
    to change the current ledger configuration. The services provides methods to modify
    different aspects of the configuration.
    """

    def GetTimeModel(self, request, context):
        """Return the currently active time model and the current configuration generation.
        Errors:
        - ``UNAUTHENTICATED``: if the request does not include a valid access token
        - ``PERMISSION_DENIED``: if the claims in the token are insufficient to perform a given operation
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SetTimeModel(self, request, context):
        """Set the ledger time model.
        Errors:
        - ``UNAUTHENTICATED``: if the request does not include a valid access token
        - ``PERMISSION_DENIED``: if the claims in the token are insufficient to perform a given operation
        - ``INVALID_ARGUMENT``: if arguments are invalid, or the provided configuration generation
        does not match the current active configuration generation. The caller is expected
        to retry by again fetching current time model using 'GetTimeModel', applying changes
        and resubmitting.
        - ``ABORTED``: if the request is rejected or times out. Note that a timed out request may
        have still been committed to the ledger. Application should re-query the current
        time model before retrying.
        - ``UNIMPLEMENTED``: if this method is not supported by the backing ledger.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_ConfigManagementServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "GetTimeModel": grpc.unary_unary_rpc_method_handler(
            servicer.GetTimeModel,
            request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.GetTimeModelRequest.FromString,
            response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.GetTimeModelResponse.SerializeToString,
        ),
        "SetTimeModel": grpc.unary_unary_rpc_method_handler(
            servicer.SetTimeModel,
            request_deserializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.SetTimeModelRequest.FromString,
            response_serializer=com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.SetTimeModelResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "com.daml.ledger.api.v1.admin.ConfigManagementService", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class ConfigManagementService(object):
    """Status: experimental interface, will change before it is deemed production
    ready

    Ledger configuration management service provides methods for the ledger administrator
    to change the current ledger configuration. The services provides methods to modify
    different aspects of the configuration.
    """

    @staticmethod
    def GetTimeModel(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/com.daml.ledger.api.v1.admin.ConfigManagementService/GetTimeModel",
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.GetTimeModelRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.GetTimeModelResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SetTimeModel(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/com.daml.ledger.api.v1.admin.ConfigManagementService/SetTimeModel",
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.SetTimeModelRequest.SerializeToString,
            com_dot_daml_dot_ledger_dot_api_dot_v1_dot_admin_dot_config__management__service__pb2.SetTimeModelResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/version_service.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/version_service.proto",
    package="com.daml.ledger.api.v1",
    syntax="proto3",
    serialized_options=b"\n\026com.daml.ledger.api.v1B\030VersionServiceOuterClass\252\002\026Com.Daml.Ledger.Api.V1",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n,com/daml/ledger/api/v1/version_service.proto\x12\x16\x63om.daml.ledger.api.v1"/\n\x1aGetLedgerApiVersionRequest\x12\x11\n\tledger_id\x18\x01 \x01(\t".\n\x1bGetLedgerApiVersionResponse\x12\x0f\n\x07version\x18\x01 \x01(\t2\x90\x01\n\x0eVersionService\x12~\n\x13GetLedgerApiVersion\x12\x32.com.daml.ledger.api.v1.GetLedgerApiVersionRequest\x1a\x33.com.daml.ledger.api.v1.GetLedgerApiVersionResponseBK\n\x16\x63om.daml.ledger.api.v1B\x18VersionServiceOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3',
)


_GETLEDGERAPIVERSIONREQUEST = _descriptor.Descriptor(
    name="GetLedgerApiVersionRequest",
    full_name="com.daml.ledger.api.v1.GetLedgerApiVersionRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="ledger_id",
            full_name="com.daml.ledger.api.v1.GetLedgerApiVersionRequest.ledger_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=72,
    serialized_end=119,
)


_GETLEDGERAPIVERSIONRESPONSE = _descriptor.Descriptor(
    name="GetLedgerApiVersionResponse",
    full_name="com.daml.ledger.api.v1.GetLedgerApiVersionResponse",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="version",
            full_name="com.daml.ledger.api.v1.GetLedgerApiVersionResponse.version",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=121,
    serialized_end=167,
)

DESCRIPTOR.message_types_by_name["GetLedgerApiVersionRequest"] = _GETLEDGERAPIVERSIONREQUEST
DESCRIPTOR.message_types_by_name["GetLedgerApiVersionResponse"] = _GETLEDGERAPIVERSIONRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

GetLedgerApiVersionRequest = _reflection.GeneratedProtocolMessageType(
    "GetLedgerApiVersionRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETLEDGERAPIVERSIONREQUEST,
        "__module__": "com.daml.ledger.api.v1.version_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerApiVersionRequest)
    },
)
_sym_db.RegisterMessage(GetLedgerApiVersionRequest)

GetLedgerApiVersionResponse = _reflection.GeneratedProtocolMessageType(
    "GetLedgerApiVersionResponse",
    (_message.Message,),
    {
        "DESCRIPTOR": _GETLEDGERAPIVERSIONRESPONSE,
        "__module__": "com.daml.ledger.api.v1.version_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.GetLedgerApiVersionResponse)
    },
)
_sym_db.RegisterMessage(GetLedgerApiVersionResponse)


DESCRIPTOR._options = None

_VERSIONSERVICE = _descriptor.ServiceDescriptor(
    name="VersionService",
    full_name="com.daml.ledger.api.v1.VersionService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=170,
    serialized_end=314,
    methods=[
        _descriptor.MethodDescriptor(
            name="GetLedgerApiVersion",
            full_name="com.daml.ledger.api.v1.VersionService.GetLedgerApiVersion",
            index=0,
            containing_service=None,
            input_type=_GETLEDGERAPIVERSIONREQUEST,
            output_type=_GETLEDGERAPIVERSIONRESPONSE,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_VERSIONSERVICE)

DESCRIPTOR.services_by_name["VersionService"] = _VERSIONSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from collections import defaultdict
from inspect import iscoroutine
from typing import (
    Any,
    AsyncIterator,
    Awaitable,
    Callable,
    DefaultDict,
    List,
    TypeVar,
    Union,
    overload,
)
import warnings

from .api_types import ArchiveEvent, Boundary, CreateEvent
from .errors import CallbackReturnWarning

__all__ = ["QueryStreamBase"]

CREATE_EVENT = "create"
ARCHIVE_EVENT = "archive"
BOUNDARY = "boundary"

Self = TypeVar("Self")


class QueryStreamBase:
    @property
    def _callbacks(
        self,
    ) -> "DefaultDict[str, List[Union[Callable[[Any], None], Callable[[Any], Awaitable[None]]]]]":
        cb = getattr(self, "__cb", None)
        if cb is None:
            cb = defaultdict(list)
            object.__setattr__(self, "__cb", cb)
        return cb

    @overload
    def on_boundary(self, fn: "Callable[[Boundary], None]") -> "Callable[[Boundary], None]":
        ...

    @overload
    def on_boundary(
        self, fn: "Callable[[Boundary], Awaitable[None]]"
    ) -> "Callable[[Boundary], Awaitable[None]]":
        ...

    def on_boundary(self, fn):
        if not callable(fn):
            raise ValueError("fn must be a callable")

        self._callbacks[BOUNDARY].append(fn)

    @overload
    def on_create(self, fn: "Callable[[CreateEvent], None]") -> "Callable[[CreateEvent], None]":
        ...

    @overload
    def on_create(
        self, fn: "Callable[[CreateEvent], Awaitable[None]]"
    ) -> "Callable[[CreateEvent], Awaitable[None]]":
        ...

    def on_create(self, fn):
        if not callable(fn):
            raise ValueError("fn must be a callable")

        self._callbacks[CREATE_EVENT].append(fn)

    @overload
    def on_archive(self, fn: "Callable[[ArchiveEvent], None]") -> "Callable[[ArchiveEvent], None]":
        ...

    @overload
    def on_archive(
        self, fn: "Callable[[ArchiveEvent], Awaitable[None]]"
    ) -> "Callable[[ArchiveEvent], Awaitable[None]]":
        ...

    def on_archive(self, fn):
        if not callable(fn):
            raise ValueError("fn must be a callable")

        self._callbacks[ARCHIVE_EVENT].append(fn)

    async def __aenter__(self: Self) -> "Self":
        """
        Prepare the stream.
        """
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        """
        Close the stream.
        """

    async def close(self) -> None:
        """
        Close and dispose of any resources used by this stream.
        """

    async def run(self) -> "None":
        """
        "Runs" the stream. This can be called as an alternative to :meth:`items` when using
        callback-based APIs.
        """
        async for _ in self:
            pass

    async def creates(self) -> "AsyncIterator[CreateEvent]":
        """
        Return a stream of :class:`CreateEvent`s. This will include the contracts of the
        Active Contract Set, as well as create events over subsequent transactions.
        """
        async for item in self.items():
            if isinstance(item, CreateEvent):
                yield item

    async def events(self) -> "AsyncIterator[Union[CreateEvent, ArchiveEvent]]":
        """
        Return a stream of :class:`CreateEvent`s. This will include the contracts of the
        Active Contract Set, as well as create and archive events over subsequent transactions.
        """
        async for item in self.items():
            if isinstance(item, CreateEvent) or isinstance(item, ArchiveEvent):
                yield item

    def items(self) -> "AsyncIterator[Union[CreateEvent, ArchiveEvent, Boundary]]":
        """
        Must be overridden by subclasses to provide a stream of events. The implementation is
        expected to call :meth:`_emit_create` and :meth:`_emit_archive` for every encountered event.
        """
        raise NotImplementedError

    def __aiter__(self) -> "AsyncIterator[Union[CreateEvent, ArchiveEvent, Boundary]]":
        """
        Returns :meth:`items`, which includes all create and archive events, and boundaries.
        """
        return self.items()

    async def _emit(self, name: str, obj: "Any"):
        for cb in self._callbacks[name]:
            result = cb(obj)
            if result is not None and iscoroutine(result):
                result = await result
            if result is not None:
                warnings.warn(
                    "callbacks should not return anything; the result will be ignored",
                    CallbackReturnWarning,
                )

    async def _emit_create(self, event: "CreateEvent"):
        await self._emit(CREATE_EVENT, event)

    async def _emit_archive(self, event: "ArchiveEvent"):
        await self._emit(ARCHIVE_EVENT, event)

    async def _emit_boundary(self, event: "Boundary"):
        await self._emit(BOUNDARY, event)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
Conversion methods from Ledger API Protobuf-generated types to dazl/Pythonic types.
"""
from dataclasses import dataclass
from datetime import datetime
from typing import Any, Dict, Iterable, List, Optional, Sequence, Union
import warnings

from ... import LOG
from ..._gen.com.daml.ledger.api.v1 import (
    active_contracts_service_pb2 as acs_pb2,
    event_pb2,
    ledger_offset_pb2 as lo_pb2,
    transaction_filter_pb2 as txf_pb2,
    transaction_pb2 as tx_pb2,
    transaction_service_pb2 as txs_pb2,
    value_pb2,
)
from ...damlast.daml_lf_1 import TypeConName
from ...damlast.daml_types import con
from ...damlast.lookup import find_choice
from ...damlast.protocols import SymbolLookup
from ...ledger.grpc.codec_aio import Codec
from ...prim import Party, to_datetime
from ...values import Context, ProtobufDecoder
from ..events import (
    ActiveContractSetEvent,
    BaseEvent,
    ContractArchiveEvent,
    ContractCreateEvent,
    ContractExercisedEvent,
    ContractFilter,
    OffsetEvent,
    TransactionEndEvent,
    TransactionFilter,
    TransactionStartEvent,
)

DECODER = ProtobufDecoder()


@dataclass(frozen=True)
class BaseEventDeserializationContext:
    """
    Attributes required throughout the deserialization of an event stream.
    """

    client: "Any"
    lookup: "SymbolLookup"
    party: "Party"
    ledger_id: str

    def deserializer_context(self) -> "Context":
        return Context(DECODER, self.lookup)

    def offset_event(self, time: datetime, offset: str) -> OffsetEvent:
        return OffsetEvent(self.client, self.party, time, self.ledger_id, self.lookup, offset)

    def active_contract_set(
        self, offset: str, workflow_id: str
    ) -> "ActiveContractSetEventDeserializationContext":
        return ActiveContractSetEventDeserializationContext(
            self.client, self.lookup, self.party, self.ledger_id, offset, workflow_id
        )

    def transaction(
        self, time: datetime, offset: str, command_id: str, workflow_id: str
    ) -> "TransactionEventDeserializationContext":
        return TransactionEventDeserializationContext(
            self.client,
            self.lookup,
            self.party,
            self.ledger_id,
            time,
            offset,
            command_id,
            workflow_id,
        )


@dataclass(frozen=True)
class ActiveContractSetEventDeserializationContext(BaseEventDeserializationContext):
    """
    Attributes required throughout the deserialization of the Active Contract Set.
    """

    offset: str
    workflow_id: str

    def active_contract_set_event(
        self, contract_events: "Sequence[ContractCreateEvent]"
    ) -> "ActiveContractSetEvent":
        return ActiveContractSetEvent(
            self.client,
            self.party,
            None,
            self.ledger_id,
            self.lookup,
            self.offset,
            contract_events,
        )

    def contract_created_event(self, cid, cdata, event_id, witness_parties) -> ContractCreateEvent:
        return ContractCreateEvent(
            client=self.client,
            party=self.party,
            time=None,
            ledger_id=self.ledger_id,
            lookup=self.lookup,
            offset=self.offset,
            command_id="",
            workflow_id=self.workflow_id,
            cid=cid,
            cdata=cdata,
            event_id=event_id,
            witness_parties=witness_parties,
        )


@dataclass(frozen=True)
class TransactionEventDeserializationContext(BaseEventDeserializationContext):
    """
    Attributes required throughout the deserialization of an event stream.
    """

    time: datetime
    offset: str
    command_id: str
    workflow_id: str

    def transaction_start_event(self, contract_events) -> TransactionStartEvent:
        return TransactionStartEvent(
            self.client,
            self.party,
            self.time,
            self.ledger_id,
            self.lookup,
            self.offset,
            self.command_id,
            self.workflow_id,
            contract_events,
        )

    def transaction_end_event(self, contract_events) -> TransactionEndEvent:
        return TransactionEndEvent(
            self.client,
            self.party,
            self.time,
            self.ledger_id,
            self.lookup,
            self.offset,
            self.command_id,
            self.workflow_id,
            contract_events,
        )

    def contract_created_event(self, cid, cdata, event_id, witness_parties) -> ContractCreateEvent:
        return ContractCreateEvent(
            client=self.client,
            party=self.party,
            time=self.time,
            ledger_id=self.ledger_id,
            lookup=self.lookup,
            offset=self.offset,
            command_id=self.command_id,
            workflow_id=self.workflow_id,
            cid=cid,
            cdata=cdata,
            event_id=event_id,
            witness_parties=witness_parties,
        )

    def contract_exercised_event(
        self,
        cid,
        cdata,
        event_id,
        witness_parties,
        contract_creating_event_id: str,
        choice: str,
        choice_argument: Any,
        acting_parties,
        consuming,
        child_event_ids,
        exercise_result,
    ) -> ContractExercisedEvent:
        return ContractExercisedEvent(
            client=self.client,
            party=self.party,
            time=self.time,
            ledger_id=self.ledger_id,
            lookup=self.lookup,
            offset=self.offset,
            command_id=self.command_id,
            workflow_id=self.workflow_id,
            cid=cid,
            cdata=cdata,
            event_id=event_id,
            witness_parties=witness_parties,
            contract_creating_event_id=contract_creating_event_id,
            acting_parties=acting_parties,
            choice=choice,
            choice_args=choice_argument,
            consuming=consuming,
            child_event_ids=child_event_ids,
            exercise_result=exercise_result,
        )

    def contract_archived_event(
        self, cid, cdata, event_id, witness_parties
    ) -> ContractArchiveEvent:
        return ContractArchiveEvent(
            client=self.client,
            party=self.party,
            time=self.time,
            ledger_id=self.ledger_id,
            lookup=self.lookup,
            offset=self.offset,
            command_id=self.command_id,
            workflow_id=self.workflow_id,
            cid=cid,
            cdata=cdata,
            event_id=event_id,
            witness_parties=witness_parties,
        )


def serialize_transactions_request(
    f: "TransactionFilter", ledger_id: str, party: str
) -> "txs_pb2.GetTransactionsRequest":
    if f.current_offset is not None:
        ledger_offset = lo_pb2.LedgerOffset()
        ledger_offset.absolute = f.current_offset
    else:
        ledger_offset = lo_pb2.LedgerOffset()
        ledger_offset.boundary = 0

    if f.destination_offset is not None:
        final_offset = lo_pb2.LedgerOffset()
        final_offset.absolute = f.destination_offset
    else:
        final_offset = lo_pb2.LedgerOffset()
        final_offset.boundary = 1

    return txs_pb2.GetTransactionsRequest(
        ledger_id=ledger_id,
        begin=ledger_offset,
        end=final_offset,
        filter=serialize_transaction_filter(f, party),
    )


def serialize_acs_request(
    f: "ContractFilter", ledger_id: str, party: str
) -> "acs_pb2.GetActiveContractsRequest":
    return acs_pb2.GetActiveContractsRequest(
        ledger_id=ledger_id, filter=serialize_transaction_filter(f, party)
    )


def serialize_event_id_request(
    ledger_id: str, event_id: str, requesting_parties: "Sequence[str]"
) -> "txs_pb2.GetTransactionByEventIdRequest":
    return txs_pb2.GetTransactionByEventIdRequest(
        ledger_id=ledger_id, event_id=event_id, requesting_parties=requesting_parties
    )


def serialize_transaction_filter(
    contract_filter: "ContractFilter", party: str
) -> "txf_pb2.TransactionFilter":
    identifiers = (
        [Codec.encode_identifier(tt) for tt in contract_filter.templates]
        if contract_filter.templates is not None
        else None
    )

    parties = [party]
    if contract_filter.party_groups is not None:
        parties.extend(contract_filter.party_groups)

    filters_by_party = {}
    for party in parties:
        if identifiers is not None:
            filters_by_party[party] = txf_pb2.Filters(
                inclusive=txf_pb2.InclusiveFilters(template_ids=identifiers)
            )
        else:
            filters_by_party[party] = txf_pb2.Filters()

    return txf_pb2.TransactionFilter(filters_by_party=filters_by_party)


def to_transaction_events(
    context: "BaseEventDeserializationContext",
    tx_stream_pb: "Iterable[txs_pb2.GetTransactionsResponse]",
    tt_stream_pb: "Optional[Iterable[txs_pb2.GetTransactionTreesResponse]]",
    last_offset_override: "Optional[str]",
) -> "Sequence[BaseEvent]":
    """
    Convert a stream of :class:`GetTransactionsResponse` into a sequence of events.

    :param context:
        Additional data required for deserializing wire information into easily-consumable
        application events.
    :param tx_stream_pb:
        A stream of :class:`GetTransactionsResponse`.
    :param tt_stream_pb:
        An optional stream of :class:`GetTransactionTreesResponse`, used to enrich the stream with
        exercise events. Transactions included in this message that are _not_ in ``tx_stream_pb``
        will be discarded.
    :param last_offset_override:
        An optional last offset, that, if specified AND different from the last offset of the
        stream, causes a synthetic :class:`OffsetEvent` to be created at the end.
    :return:
    """
    events = []  # type: List[OffsetEvent]

    events_by_offset = dict()  # type: Dict[str, List[OffsetEvent]]
    for item_pb in tx_stream_pb:
        for transaction_pb in item_pb.transactions:
            events_by_offset[transaction_pb.offset] = list(
                to_transaction_chunk(context, transaction_pb)
            )

    if tt_stream_pb is not None:
        for transaction_tree_pb in tt_stream_pb:
            for transaction_pb in transaction_tree_pb.transactions:
                tx_events = events_by_offset.get(transaction_pb.offset)
                if tx_events is not None:
                    tx_events[-1:-1] = from_transaction_tree(context, transaction_pb)

    for tx_events in events_by_offset.values():
        events.extend(tx_events)

    if last_offset_override is not None:
        last_time = None if not events else events[-1].time
        last_offset = None if not events else events[-1].offset
        if last_offset != last_offset_override:
            events.append(context.offset_event(last_time, last_offset_override))

    return events


def to_acs_events(
    context: "BaseEventDeserializationContext", acs_stream_pb: "Iterable[Any]"
) -> "Sequence[ActiveContractSetEvent]":
    return [
        acs_evt
        for acs_response_pb in acs_stream_pb
        for acs_evt in to_acs_event(context, acs_response_pb)
    ]


def from_transaction_tree(
    context: "BaseEventDeserializationContext", tt_pb: "tx_pb2.TransactionTree"
) -> "Sequence[OffsetEvent]":
    t_context = context.transaction(
        time=to_datetime(tt_pb.effective_at),
        offset=tt_pb.offset,
        command_id=tt_pb.command_id,
        workflow_id=tt_pb.workflow_id,
    )

    events = []
    for evt_pb in tt_pb.events_by_id.values():
        evt = to_event(t_context, evt_pb)
        if isinstance(evt, ContractExercisedEvent):
            events.append(evt)

    return events


def to_acs_event(
    context: "BaseEventDeserializationContext", acs_pb: "acs_pb2.GetActiveContractsResponse"
):
    acs_context = context.active_contract_set(acs_pb.offset, acs_pb.workflow_id)
    contract_events = [
        evt
        for evt in (to_created_event(acs_context, evt_pb) for evt_pb in acs_pb.active_contracts)
        if evt is not None
    ]
    return [acs_context.active_contract_set_event(contract_events)]


def to_transaction_chunk(
    context: "BaseEventDeserializationContext", tx_pb: "tx_pb2.Transaction"
) -> "Sequence[OffsetEvent]":
    """
    Return a sequence of events parsed from a Ledger API ``Transaction`` protobuf message.

    :param context:
        Additional information taken from the enclosing message.
    :param tx_pb:
        The Ledger API ``Transaction`` to parse.
    :return:
        A :class:`TransactionStartEvent`, followed by zero or more :class:`ContractCreateEvent`
        and/or :class:`ContractArchiveEvent`, followed by :class:`TransactionEndEvent`.
    """

    t_context = context.transaction(
        time=to_datetime(tx_pb.effective_at),
        offset=tx_pb.offset,
        command_id=tx_pb.command_id,
        workflow_id=tx_pb.workflow_id,
    )

    contract_events = [to_event(t_context, evt_pb) for evt_pb in tx_pb.events]
    contract_events = [evt for evt in contract_events if evt is not None]

    return [
        t_context.transaction_start_event(contract_events),
        *contract_events,
        t_context.transaction_end_event(contract_events),
    ]


def to_event(
    context: "Union[TransactionEventDeserializationContext, ActiveContractSetEventDeserializationContext]",
    evt_pb: "event_pb2.Event",
) -> "Optional[BaseEvent]":
    try:
        event_type = evt_pb.WhichOneof("event")
    except ValueError:
        try:
            event_type = evt_pb.WhichOneof("kind")
        except ValueError:
            LOG.error("Deserialization error into an event of %r", evt_pb)
            raise

    if "created" == event_type:
        return to_created_event(context, evt_pb.created)
    elif "exercised" == event_type:
        return to_exercised_event(context, evt_pb.exercised)
    elif "archived" == event_type:
        return to_archived_event(context, evt_pb.archived)
    else:
        raise ValueError(f"unknown event type: {event_type}")


def to_created_event(
    context: "Union[TransactionEventDeserializationContext, ActiveContractSetEventDeserializationContext]",
    cr: "event_pb2.CreatedEvent",
) -> "Optional[ContractCreateEvent]":
    tt = con(Codec.decode_identifier(cr.template_id))

    ctx = context.deserializer_context()
    cid = ctx.convert_contract_id(tt, cr.contract_id)
    cdata = ctx.convert(tt, cr.create_arguments)
    event_id = cr.event_id
    witness_parties = tuple(cr.witness_parties)

    return context.contract_created_event(cid, cdata, event_id, witness_parties)


def to_exercised_event(
    context: "TransactionEventDeserializationContext", er: "event_pb2.ExercisedEvent"
) -> "Optional[ContractExercisedEvent]":
    name = Codec.decode_identifier(er.template_id)
    tt = con(name)

    template = context.lookup.template(name)
    choice = find_choice(template, er.choice)

    ctx = context.deserializer_context()

    cid = ctx.convert_contract_id(tt, er.contract_id)
    event_id = er.event_id
    witness_parties = tuple(er.witness_parties)
    try:
        contract_creating_event_id = er.contract_creating_event_id
    except AttributeError:
        contract_creating_event_id = None
    choice_args = ctx.convert(choice.arg_binder.type, er.choice_argument)
    acting_parties = tuple(er.acting_parties)
    consuming = er.consuming
    child_event_ids = er.child_event_ids
    exercise_result = ctx.convert(choice.ret_type, er.exercise_result)

    return context.contract_exercised_event(
        cid,
        None,
        event_id,
        witness_parties,
        contract_creating_event_id,
        er.choice,
        choice_args,
        acting_parties,
        consuming,
        child_event_ids,
        exercise_result,
    )


def to_archived_event(
    context: "TransactionEventDeserializationContext", ar: "event_pb2.ArchivedEvent"
) -> "Optional[ContractArchiveEvent]":
    tt = con(Codec.decode_identifier(ar.template_id))
    event_id = ar.event_id
    witness_parties = tuple(ar.witness_parties)

    ctx = context.deserializer_context()
    cid = ctx.convert_contract_id(tt, ar.contract_id)
    return context.contract_archived_event(cid, None, event_id, witness_parties)


def to_type_con_name(identifier: "value_pb2.Identifier") -> "TypeConName":
    warnings.warn("Use Codec.decode_identifier instead.", DeprecationWarning, stacklevel=2)
    return Codec.decode_identifier(identifier)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/ledger_offset.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/ledger_offset.proto",
    package="com.daml.ledger.api.v1",
    syntax="proto3",
    serialized_options=b"\n\026com.daml.ledger.api.v1B\026LedgerOffsetOuterClass\252\002\026Com.Daml.Ledger.Api.V1",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n*com/daml/ledger/api/v1/ledger_offset.proto\x12\x16\x63om.daml.ledger.api.v1"\xa8\x01\n\x0cLedgerOffset\x12\x12\n\x08\x61\x62solute\x18\x01 \x01(\tH\x00\x12G\n\x08\x62oundary\x18\x02 \x01(\x0e\x32\x33.com.daml.ledger.api.v1.LedgerOffset.LedgerBoundaryH\x00"2\n\x0eLedgerBoundary\x12\x10\n\x0cLEDGER_BEGIN\x10\x00\x12\x0e\n\nLEDGER_END\x10\x01\x42\x07\n\x05valueBI\n\x16\x63om.daml.ledger.api.v1B\x16LedgerOffsetOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3',
)


_LEDGEROFFSET_LEDGERBOUNDARY = _descriptor.EnumDescriptor(
    name="LedgerBoundary",
    full_name="com.daml.ledger.api.v1.LedgerOffset.LedgerBoundary",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="LEDGER_BEGIN",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="LEDGER_END",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=180,
    serialized_end=230,
)
_sym_db.RegisterEnumDescriptor(_LEDGEROFFSET_LEDGERBOUNDARY)


_LEDGEROFFSET = _descriptor.Descriptor(
    name="LedgerOffset",
    full_name="com.daml.ledger.api.v1.LedgerOffset",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="absolute",
            full_name="com.daml.ledger.api.v1.LedgerOffset.absolute",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="boundary",
            full_name="com.daml.ledger.api.v1.LedgerOffset.boundary",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _LEDGEROFFSET_LEDGERBOUNDARY,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="value",
            full_name="com.daml.ledger.api.v1.LedgerOffset.value",
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=71,
    serialized_end=239,
)

_LEDGEROFFSET.fields_by_name["boundary"].enum_type = _LEDGEROFFSET_LEDGERBOUNDARY
_LEDGEROFFSET_LEDGERBOUNDARY.containing_type = _LEDGEROFFSET
_LEDGEROFFSET.oneofs_by_name["value"].fields.append(_LEDGEROFFSET.fields_by_name["absolute"])
_LEDGEROFFSET.fields_by_name["absolute"].containing_oneof = _LEDGEROFFSET.oneofs_by_name["value"]
_LEDGEROFFSET.oneofs_by_name["value"].fields.append(_LEDGEROFFSET.fields_by_name["boundary"])
_LEDGEROFFSET.fields_by_name["boundary"].containing_oneof = _LEDGEROFFSET.oneofs_by_name["value"]
DESCRIPTOR.message_types_by_name["LedgerOffset"] = _LEDGEROFFSET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LedgerOffset = _reflection.GeneratedProtocolMessageType(
    "LedgerOffset",
    (_message.Message,),
    {
        "DESCRIPTOR": _LEDGEROFFSET,
        "__module__": "com.daml.ledger.api.v1.ledger_offset_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.LedgerOffset)
    },
)
_sym_db.RegisterMessage(LedgerOffset)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

"""
This module has been relocated to ``dazl.client.events`` or ``dazl.damlast.lookup``.
"""

__all__ = ["validate_template", "template_reverse_globs"]

from ..client.events import template_reverse_globs
from ..damlast.lookup import validate_template



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: com/daml/ledger/api/v1/command_submission_service.proto

from google.protobuf import (
    descriptor as _descriptor,
    message as _message,
    reflection as _reflection,
    symbol_database as _symbol_database,
)

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2

from . import (
    commands_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_commands__pb2,
    trace_context_pb2 as com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2,
)

DESCRIPTOR = _descriptor.FileDescriptor(
    name="com/daml/ledger/api/v1/command_submission_service.proto",
    package="com.daml.ledger.api.v1",
    syntax="proto3",
    serialized_options=b'\n\026com.daml.ledger.api.v1B"CommandSubmissionServiceOuterClass\252\002\026Com.Daml.Ledger.Api.V1',
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n7com/daml/ledger/api/v1/command_submission_service.proto\x12\x16\x63om.daml.ledger.api.v1\x1a*com/daml/ledger/api/v1/trace_context.proto\x1a%com/daml/ledger/api/v1/commands.proto\x1a\x1bgoogle/protobuf/empty.proto"\x81\x01\n\rSubmitRequest\x12\x32\n\x08\x63ommands\x18\x01 \x01(\x0b\x32 .com.daml.ledger.api.v1.Commands\x12<\n\rtrace_context\x18\xe8\x07 \x01(\x0b\x32$.com.daml.ledger.api.v1.TraceContext2c\n\x18\x43ommandSubmissionService\x12G\n\x06Submit\x12%.com.daml.ledger.api.v1.SubmitRequest\x1a\x16.google.protobuf.EmptyBU\n\x16\x63om.daml.ledger.api.v1B"CommandSubmissionServiceOuterClass\xaa\x02\x16\x43om.Daml.Ledger.Api.V1b\x06proto3',
    dependencies=[
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2.DESCRIPTOR,
        com_dot_daml_dot_ledger_dot_api_dot_v1_dot_commands__pb2.DESCRIPTOR,
        google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,
    ],
)


_SUBMITREQUEST = _descriptor.Descriptor(
    name="SubmitRequest",
    full_name="com.daml.ledger.api.v1.SubmitRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="commands",
            full_name="com.daml.ledger.api.v1.SubmitRequest.commands",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="trace_context",
            full_name="com.daml.ledger.api.v1.SubmitRequest.trace_context",
            index=1,
            number=1000,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=196,
    serialized_end=325,
)

_SUBMITREQUEST.fields_by_name[
    "commands"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_commands__pb2._COMMANDS
_SUBMITREQUEST.fields_by_name[
    "trace_context"
].message_type = com_dot_daml_dot_ledger_dot_api_dot_v1_dot_trace__context__pb2._TRACECONTEXT
DESCRIPTOR.message_types_by_name["SubmitRequest"] = _SUBMITREQUEST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SubmitRequest = _reflection.GeneratedProtocolMessageType(
    "SubmitRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _SUBMITREQUEST,
        "__module__": "com.daml.ledger.api.v1.command_submission_service_pb2"
        # @@protoc_insertion_point(class_scope:com.daml.ledger.api.v1.SubmitRequest)
    },
)
_sym_db.RegisterMessage(SubmitRequest)


DESCRIPTOR._options = None

_COMMANDSUBMISSIONSERVICE = _descriptor.ServiceDescriptor(
    name="CommandSubmissionService",
    full_name="com.daml.ledger.api.v1.CommandSubmissionService",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=327,
    serialized_end=426,
    methods=[
        _descriptor.MethodDescriptor(
            name="Submit",
            full_name="com.daml.ledger.api.v1.CommandSubmissionService.Submit",
            index=0,
            containing_service=None,
            input_type=_SUBMITREQUEST,
            output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_COMMANDSUBMISSIONSERVICE)

DESCRIPTOR.services_by_name["CommandSubmissionService"] = _COMMANDSUBMISSIONSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0


from datetime import timedelta

from ..prim import Party

__all__ = ["MetricEvents"]


class MetricEvents:
    def loop_responsiveness(self, responsiveness: "timedelta") -> None:
        """
        Record the length of time it takes to immediately schedule and execute a no-op on the event
        loop. This measurement can be used to get a sense of how "backed up" the current event loop
        is.

        :param responsiveness:
            The length of time between the scheduling of an action and the invocation of it.
        """

    def party_offset(self, party: "Party", offset: str) -> None:
        """
        Record that a party has reached a certain block offset.

        :param party: The party that has reached an offset.
        :param offset: The offset that has been reached.
        """



# Copyright (c) 2017-2021 Digital Asset (Switzerland) GmbH and/or its affiliates. All rights reserved.
# SPDX-License-Identifier: Apache-2.0

from typing import Any, Mapping

from ..damlast.daml_lf_1 import TypeConName

__all__ = ["ContractId", "ContractData"]


class ContractId:
    """
    A typed contract ID.

    Instance attributes:

    .. attribute:: ContractId.value

        The raw contract ID value (for example, ``"#4:1"``).

    .. attribute:: ContractId.value_type

        The type of template that is pointed to by this :class:`ContractId`.

    """

    __slots__ = "value", "value_type"

    def __init__(self, value_type: "TypeConName", value: str):
        if not isinstance(value_type, TypeConName):
            raise ValueError("value_type must be a TypeConName")
        if not isinstance(value, str):
            raise ValueError("value must be a string")

        object.__setattr__(self, "value_type", value_type)
        object.__setattr__(self, "value", value)

    def __str__(self):
        """
        Return the contract ID without a type adornment.
        """
        return self.value

    def __repr__(self):
        """
        Return a more detailed view of the ContractId.
        """
        return f"ContractId({self.value_type}, {self.value!r})"

    def __eq__(self, other):
        """
        Returns whether this contract is the same as the other one. Template
        type is NOT considered in equality.
        """
        return isinstance(other, ContractId) and self.value == other.value

    def __format__(self, format_spec):
        """
        Format the ContractId according to the spec.
        """
        return ("{:" + format_spec + "s}").format(self.value)

    def __hash__(self):
        """
        Returns a hash of the ContractId (based on the value of ContractId).
        """
        return hash(self.value)

    def __setattr__(self, key, value):
        """
        Overridden to make ContractId instances read-only.
        """
        raise AttributeError("ContractId instances are read-only")


ContractData = Mapping[str, Any]
